;;;;;
;;;;;		#=============================================#
;;;;;		||					     ||
;;;;;		||		   MCGA LIBRARY		     ||
;;;;;		||	   (C) 1992, 1993  NILS M. HOLM	     ||
;;;;;		||					     ||
;;;;;		#=============================================#
;;;;;

DOS_SERVICES	equ	21h
BIOS_VIDEO	equ	10h
MCGA_COLS	equ	320
MCGA_VRAM	equ	0a000h


DGROUP	GROUP	_DATA, _BSS

_DATA	SEGMENT WORD PUBLIC	'DATA'

	PUBLIC	__mcga_vram

__COPYRIGHT	db	"MCGA GRAFIX LIBRARY -- V2.0 -- "
		db	"COPYRIGHT (C) 1992-1994 NILS M. HOLM"
__bb_oldmode	dw	3
__bb_csrshape	dw	0b0ch
__bb_init	db	0
__bb_noinit	db	"mcga mode not initialized, cannot reset mode!"
		db	13, 10, "$"
__mcga_vram	dw	MCGA_VRAM

_DATA	ENDS


_BSS	SEGMENT WORD PUBLIC	'BSS'
_BSS	ENDS


_TEXT	SEGMENT BYTE PUBLIC	'CODE'
	ASSUME CS:_TEXT, DS:DGROUP
	PUBLIC	_initmcga, _endmcga, _mcgapixel, _mcgabox, _mcgaclear
	PUBLIC	_mcgaputbb, _mcgahline, _mcgavline, _mcgaovlbb, _mcgagetbb
	PUBLIC	_mcgagetpx

;;;;;
;;;;;	int initmcga();
;;;;;	
;;;;;	save old video mode and switch to mcga 320x200x256 mode (0x13)
;;;;;	returns 0 on success, -1 in case of an error (no MCG Adapter).
;;;;;

_initmcga	PROC FAR
	push	si
	push	di

	mov	ax,0300h
	xor	bx,bx
	int	BIOS_VIDEO
	mov	__bb_csrshape,cx

	mov	ax,0f00h
	int	BIOS_VIDEO
	mov	__bb_oldmode,ax

	mov	ax,13h
	int	BIOS_VIDEO

	mov	ax,0f00h
	int	BIOS_VIDEO
	cmp	al,13h
	jz	bb_init_ok
	mov	ax,0ffffh
	jmp	bb_init_exit
bb_init_ok:
	mov	__bb_init,1
	xor	ax,ax

bb_init_exit:
	pop	di
	pop	si
	retf
_initmcga	ENDP


;;;;;
;;;;;	void endmcga();
;;;;;	
;;;;;	restore old video mode (saved by initmcga())
;;;;;

_endmcga	PROC FAR
	push	si
	push	di

	cmp	__bb_init,1
	jz	_do_end

	push	ds
	mov	ax,DGROUP
	mov	ds,ax
	mov	dx,offset __bb_noinit
	mov	ah,9
	int	DOS_SERVICES
	pop	ds
	pop	di
	pop	si
	retf

_do_end:
	mov	ax,__bb_oldmode
	xor	ah,ah
	int	BIOS_VIDEO

	mov	ax,0100h
	mov	cx,__bb_csrshape
	int	BIOS_VIDEO

	pop	di
	pop	si
	retf
_endmcga	ENDP


;;;;;
;;;;;	void mcgapixel(int x, int y, int color);
;;;;;	
;;;;;	write one byte to the mcga video ram (draw a pixel).
;;;;;	0 <= x <= 319,  0 <= y <= 199,   0 <= color <= 255
;;;;;

_mcgapixel	PROC FAR
	push	bp
	mov	bp,sp
	push	si
	push	es

	mov	ax,[bp+8]
	xor	dx,dx
	mov	cx,MCGA_COLS
	mul	cx
	add	ax,[bp+6]
	mov	si,ax
	mov	bx,__mcga_vram
	mov	es,bx
	mov	ax,[bp+10]
	mov	es:[si],al

	pop	es
	pop	si
	pop	bp
	retf
_mcgapixel	ENDP


;;;;;
;;;;;	int mcgagetpx(int x, int y);
;;;;;	
;;;;;	read one byte from the mcga video ram (get color of a pixel).
;;;;;	0 <= x <= 319,  0 <= y <= 199,   0 <= color <= 255
;;;;;

_mcgagetpx	PROC FAR
	push	bp
	mov	bp,sp
	push	si
	push	es

	mov	ax,[bp+8]
	xor	dx,dx
	mov	cx,MCGA_COLS
	mul	cx
	add	ax,[bp+6]
	mov	si,ax
	mov	bx,__mcga_vram
	mov	es,bx
	mov	al,es:[si]
	xor	ah,ah

	pop	es
	pop	si
	pop	bp
	retf
_mcgagetpx	ENDP


;;;;;
;;;;;	void mcgahline(int x, int y, int dx, int color);
;;;;;
;;;;;	draw a horizontal line with 'color'. (x,y) = left edge
;;;;;

_mcgahline	PROC FAR
	push	bp
	mov	bp,sp
	push	si
	push	di
	push	es

	mov	ax,__mcga_vram
	mov	es,ax

	mov	ax,[bp+8]
	mov	cx,MCGA_COLS
	mul	cx
	add	ax,[bp+6]
	mov	di,ax

	mov	cx,[bp+10]
	sub	cx,[bp+6]
	inc	cx

	mov	ax,[bp+12]

	cld

	rep stos BYTE PTR [di]

	pop	es
	pop	di
	pop	si
	pop	bp
	retf
_mcgahline	ENDP


;;;;;
;;;;;	void mcgavline(int x, int y, int dy, int color);
;;;;;
;;;;;	draw a vertical line with 'color'. (x,y) = upper edge
;;;;;

_mcgavline	PROC FAR
	push	bp
	mov	bp,sp
	push	si
	push	di
	push	es

	mov	ax,__mcga_vram
	mov	es,ax

	mov	ax,[bp+8]
	mov	cx,MCGA_COLS
	mul	cx
	add	ax,[bp+6]
	mov	di,ax

	mov	cx,[bp+10]
	sub	cx,[bp+8]
	inc	cx

	mov	ax,[bp+12]

	cld

nextvdot:
	stos	BYTE PTR [di]
	add	di,MCGA_COLS-1
	loop	nextvdot

	pop	es
	pop	di
	pop	si
	pop	bp
	retf
_mcgavline	ENDP


;;;;;
;;;;;	void mcgabox(int x, int y, int dx, int dy, int color);
;;;;;
;;;;;	draw a box and fill it with 'color'. (x,y) = left upper
;;;;;	edge,  (dx, dy) = right lower edge.
;;;;;

_mcgabox	PROC FAR
	push	bp
	mov	bp,sp
	push	si
	push	di
	push	es

	mov	ax,__mcga_vram
	mov	es,ax

	mov	ax,[bp+8]
	mov	cx,MCGA_COLS
	mul	cx
	add	ax,[bp+6]
	mov	si,ax

	mov	bx,[bp+12]
	sub	bx,[bp+8]
	inc	bx

	mov	dx,[bp+10]
	sub	dx,[bp+6]
	inc	dx

	mov	ax,[bp+14]

	cld

nextrow:
	mov	di,si
	mov	cx,dx
	rep stos BYTE PTR [di]

	add	si,MCGA_COLS
	mov	cx,bx
	dec	bx
	loop	nextrow

	pop	es
	pop	di
	pop	si
	pop	bp
	retf
_mcgabox	ENDP


;;;;;
;;;;;	void mcgaclear(int color);
;;;;;
;;;;;	clears the screen (fills the mcga VRAM with 'color').
;;;;;

_mcgaclear	PROC FAR
	push	bp
	mov	bp,sp
	push	es
	push	di

	mov	ax,__mcga_vram
	mov	es,ax
	mov	cx,32000
	xor	di,di
	mov	ax,[bp+6]
	mov	ah,al

	cld
	rep stos WORD PTR [di]

	pop	di
	pop	es
	pop	bp
	retf
_mcgaclear	ENDP


;;;;;
;;;;;	void mcgaputbb(int x, int y, BITBLK *bitb);
;;;;;
;;;;;	copies a bit block to the mcga video ram with the left upper edge
;;;;;	at (x, y).
;;;;;
;;;;;	The structure BITBLK is defined this way:
;;;;;
;;;;;	typedef struct bitblk {
;;;;;		int	x, y, init;
;;;;;		char	*stream;
;;;;;	} BITBLK;
;;;;;

_mcgaputbb	PROC FAR
	push	bp
	mov	bp,sp
	push	si
	push	di
	push	es
	push	ds

	mov	ax,[bp+8]
	mov	cx,MCGA_COLS
	mul	cx
	add	ax,[bp+6]
	mov	si,ax

	les	bx,[bp+10]

	mov	dx,es:[bx]
	inc	bx
	inc	bx
	mov	ax,es:[bx]
	add	bx,4

	les	bx,es:[bx]
	mov	bp,si			;; bp = screen offset
	mov	si,bx			;; si = stream offset
	mov	bx,dx			;; bx = rows
	mov	dx,ax			;; dx = columns

	mov	ax,__mcga_vram
	push	es
	pop	ds
	mov	es,ax

	cld

nextbbrow:
	mov	di,bp
	mov	cx,dx

nextdot:
	lods	BYTE PTR [si]
	stos	BYTE PTR [di]
	loop	nextdot

	add	bp,MCGA_COLS
	mov	cx,bx
	dec	bx
	loop	nextbbrow

	pop	ds
	pop	es
	pop	di
	pop	si
	pop	bp
	retf
_mcgaputbb	ENDP


;;;;;
;;;;;	void mcgaovlbb(int x, int y, int bgrd, BITBLK *bitb);
;;;;;
;;;;;	copies a bit block to the mcga video ram with the left upper edge
;;;;;	at (x, y). Does not overwrite background dots (non-destructive copy).
;;;;;

_mcgaovlbb	PROC FAR
	push	bp
	mov	bp,sp
	push	si
	push	di
	push	es
	push	ds

	mov	ax,[bp+8]
	mov	cx,MCGA_COLS
	mul	cx
	add	ax,[bp+6]
	mov	si,ax

	mov	ax,[bp+10]
	mov	ah,al
	push	ax

	les	bx,[bp+12]

	mov	dx,es:[bx]
	inc	bx
	inc	bx
	mov	ax,es:[bx]
	add	bx,4

	les	bx,es:[bx]
	mov	bp,si			;; bp = screen offset
	mov	si,bx			;; si = stream offset
	mov	bx,dx			;; bx = rows
	mov	dx,ax			;; dx = columns

	mov	cx,__mcga_vram
	push	es
	pop	ds
	mov	es,cx

	pop	ax

	cld

nextobrow:
	mov	di,bp
	mov	cx,dx

nextodot:
	lods	BYTE PTR [si]
	cmp	al,ah
	jne	write
	mov	al,es:[di]
write:
	stos	BYTE PTR [di]
	loop	nextodot

	add	bp,MCGA_COLS
	mov	cx,bx
	dec	bx
	loop	nextobrow

	pop	ds
	pop	es
	pop	di
	pop	si
	pop	bp
	retf
_mcgaovlbb	ENDP


;;;;;	void mcgagetbb(int x, int y, BITBLK *bitb);
;;;;;
;;;;;	copies a bit block from the mcga video ram with the left upper edge
;;;;;	at (x, y).
;;;;;

_mcgagetbb	PROC FAR
	push	bp
	mov	bp,sp
	push	si
	push	di
	push	es
	push	ds

	mov	ax,[bp+8]
	mov	cx,MCGA_COLS
	mul	cx
	add	ax,[bp+6]
	mov	si,ax

	les	bx,[bp+10]

	mov	dx,es:[bx]
	inc	bx
	inc	bx
	mov	ax,es:[bx]
	add	bx,4

	les	bx,es:[bx]
	mov	bp,si			;; bp = screen offset
	mov	di,bx			;; si = stream offset
	mov	bx,dx			;; bx = rows
	mov	dx,ax			;; dx = columns

	mov	ax,__mcga_vram
	mov	ds,ax

	cld

nextgbbrow:
	mov	si,bp
	mov	cx,dx

nextgdot:
	lods	BYTE PTR [si]
	stos	BYTE PTR [di]
	loop	nextgdot

	add	bp,MCGA_COLS
	mov	cx,bx
	dec	bx
	loop	nextgbbrow

	pop	ds
	pop	es
	pop	di
	pop	si
	pop	bp
	retf
_mcgagetbb	ENDP


_TEXT	ENDS
	END
